#version 420 core
#extension GL_ARB_explicit_uniform_location : enable

#define VERTEX_ATTRIB_POSITION 0
#define VERTEX_ATTRIB_UV 1
#define VERTEX_ATTRIB_NORMAL 2
#define VERTEX_ATTRIB_MATERIAL 3

#define UNIFORM_MVP_XFORM 0
#define UNIFORM_MV_XFORM 1
#define UNIFORM_NORMAL_XFORM 2

in layout (location = VERTEX_ATTRIB_POSITION) vec3 position; // model space position
in layout (location = VERTEX_ATTRIB_NORMAL) vec3 normal; // model space normal
in layout (location = VERTEX_ATTRIB_UV) vec2 uv; // model space uv
in layout (location = VERTEX_ATTRIB_MATERIAL) uint material; // material index

out block
{
    vec3 eyePosition; // eye space position
    vec3 eyeNormal;  // eye space normal
    vec2 uv; // uv (the same for all textures)
    flat uint material; // material index
} Out;

// Transformation matrices
layout (location = UNIFORM_MVP_XFORM) uniform mat4x4 mvpXform;
layout (location = UNIFORM_MV_XFORM) uniform mat4x4 mvXform;
layout (location = UNIFORM_NORMAL_XFORM) uniform mat3x3 normalXform;

void main()
{
    // Model position
    vec4 position = vec4(position.x, position.y, position.z, 1.0f);

    // Write gl_Position
    gl_Position = mvpXform * position;
    
    // Write fragment shader inputs
    Out.eyePosition = (mvXform * position).xyz;
    Out.eyeNormal = normalize(normalXform * normal);  // Transform and normalize the normal
    Out.uv = uv;
    Out.material = material;
}