#version 420 core
#extension GL_ARB_explicit_uniform_location : enable
#extension GL_NV_gpu_shader5 : require
#extension GL_NV_shader_buffer_load : require

#define VERTEX_ATTRIB_POSITION 0
#define VERTEX_ATTRIB_UV 1
#define VERTEX_ATTRIB_NORMAL 2

#define UNIFORM_MVP_XFORM 0
#define UNIFORM_MV_XFORM 1
#define UNIFORM_NORMAL_XFORM 2
#define UNIFORM_MATERIAL_IDS 13

in layout (location = VERTEX_ATTRIB_POSITION) vec3 position; // model space position
in layout (location = VERTEX_ATTRIB_NORMAL) vec3 normal; // model space normal
in layout (location = VERTEX_ATTRIB_UV) vec2 uv; // model space uv

out block
{
    vec3 eyePosition; // eye space position
    vec3 eyeNormal;  // eye space normal
    vec2 uv; // uv (the same for all textures)
    flat uint material; // material index
} Out;

// Transformation matrices
layout (location = UNIFORM_MVP_XFORM) uniform mat4x4* mvpXforms;
layout (location = UNIFORM_MV_XFORM) uniform mat4x4* mvXforms;
layout (location = UNIFORM_NORMAL_XFORM) uniform mat3x3* normalXforms;

// Material ids
layout (location = UNIFORM_MATERIAL_IDS) uniform uint* materialIds;

void main()
{
    // Model position
    vec4 position = vec4(position.x, position.y, position.z, 1.0f);

    // Write gl_Position
    gl_Position = mvpXforms[gl_InstanceID] * position;
    
    // Write fragment shader inputs
    Out.eyePosition = (mvXforms[gl_InstanceID] * position).xyz;
    Out.eyeNormal = normalize(normalXforms[gl_InstanceID] * normal);  // Transform and normalize the normal
    Out.uv = uv;
    Out.material = materialIds[gl_InstanceID];
}