// BindlessBuffer.cpp

#include "stdafx.h"

// Constructor
BindlessBuffer::BindlessBuffer()
    : m_buffer(0)
    , m_gpuAddress(0)
    , m_size(0)
{

}

// Destructor
BindlessBuffer::~BindlessBuffer()
{
    Destroy();
}

// Buffer
GLuint BindlessBuffer::Buffer() const
{
    return m_buffer;
}

// GpuAddress
GLuint64EXT BindlessBuffer::GpuAddress() const
{
    return m_gpuAddress;
}

// Size
GLsizeiptr BindlessBuffer::Size() const
{
    return m_size;
}

// Create
bool BindlessBuffer::Create(GLsizeiptr size, const GLvoid* data, GLenum usage)
{
    Destroy();

    glGenBuffers(1, &m_buffer);
    glNamedBufferDataEXT(m_buffer, size, data, usage);
    glGetNamedBufferParameterui64vNV(m_buffer, GL_BUFFER_GPU_ADDRESS_NV, &m_gpuAddress);
    m_size = size;

    if (glGetError() != GL_NO_ERROR) {
        Destroy();
        return false;
    }

    return true;
}

// CreateAndMakeResident
bool BindlessBuffer::CreateAndMakeResident(GLsizeiptr size, const GLvoid* data, GLenum usage, GLenum access)
{
    if (!Create(size, data, usage)) {
        return false;
    }

    MakeResident(access);

    return true;
}

// MakeResident
void BindlessBuffer::MakeResident(GLenum access) const
{
    glMakeNamedBufferResidentNV(m_buffer, access);
}

// MakeNonResident
void BindlessBuffer::MakeNonResident() const
{
    glMakeNamedBufferNonResidentNV(m_buffer);
}

// IsSupported
bool BindlessBuffer::IsSupported()
{
    GLint numExtensions = 0;
    glGetIntegerv(GL_NUM_EXTENSIONS, &numExtensions);
    if (numExtensions == 0) {
        return false;
    }

    for (GLint extension = 0; extension < numExtensions; extension++) {
        if (strcmp((const char*)glGetStringi(GL_EXTENSIONS, extension), "GL_NV_gpu_shader5") == 0) {
            return true;
        }
    }

    return false;
}

// Destroy
void BindlessBuffer::Destroy()
{
    if (m_buffer) {
        glDeleteBuffers(1, &m_buffer);
        m_buffer = 0;
        m_gpuAddress = 0;
        m_size = 0;
    }
}