// BindlessMesh.h

#pragma once

using namespace std;

class BindlessMesh
{
public:
    // Constructor and destructor
    BindlessMesh();
    ~BindlessMesh();

    // Create a bindless buffer (owned by the mesh)
    bool CreateBindlessBuffer(BindlessBuffer*& pBindlessBuffer, GLsizeiptr size, const GLvoid* data, GLenum usage);

    // Add / remove attributes from the mesh
    bool AddAttribute(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, GLuint64EXT address, GLsizeiptr length);
    bool AddAttributeI(GLuint index, GLint size, GLenum type, GLsizei stride, GLuint64EXT address, GLsizeiptr length);
    bool RemoveAttribute(GLuint index);

    // Specify vertex attributes
    void VertexAttribs() const;

    // Residency (for buffers owned by the mesh)
    void MakeResident(GLenum access) const;
    void MakeNonResident() const;

    // Enable / Disable
    void Enable() const;
    void Disable() const;

    // Primitive
    void Primitive(GLenum mode, GLuint vertices);

    // Draw
    void Draw() const;
    void DrawInstanced(GLuint instanceCount) const;

    // Transform
    glm::mat4x4& Transform();

    // IsSupported
    static bool IsSupported();

private:
    // Bindless buffers owned by the mesh
    vector<BindlessBuffer*> m_bindlessBuffers;

    // Attribute structure
    struct Attrib {
        GLint size;
        GLenum type;
        GLboolean normalized;
        GLsizei stride;
        GLuint64EXT address;
        GLsizeiptr length;
        GLenum baseType;
    };

    // Attributes
    map<GLuint, Attrib> m_attribs;

    // Primitive description
    GLenum m_mode;
    GLenum m_vertices;

    // Transform associated with the mesh
    glm::mat4x4 m_transform;
};

// Populate a mesh with a screen covering triangle
bool PopulateScreenCoveringTriangleMesh(BindlessMesh& mesh);