// BindlessTexture.cpp

#include "stdafx.h"

// Constructor
BindlessTexture::BindlessTexture()
    : m_texture(0)
    , m_handle(0)
    , m_target(GL_NONE)
{

}

// Destructor
BindlessTexture::~BindlessTexture()
{
    Destroy();
}

// Texture
GLuint BindlessTexture::Texture() const
{
    return m_texture;
}

// Handle
GLuint64EXT BindlessTexture::Handle() const
{
    return m_handle;
}

// Target
GLenum BindlessTexture::Target() const
{
    return m_target;
}

// Create
bool BindlessTexture::Create2D(GLenum target, GLuint levels, GLenum format, GLuint width, GLuint height)
{
    Destroy();

    glGenTextures(1, &m_texture);
    glTextureStorage2DEXT(m_texture, target, levels, format, width, height);
    m_handle = glGetTextureHandleNV(m_texture);

    if (glGetError() != GL_NO_ERROR) {
        Destroy();
        return false;
    }

    m_target = target;

    return true;
}

// CreateAndMakeResident
bool BindlessTexture::Create2DAndMakeResident(GLenum target, GLuint levels, GLenum format, GLuint width, GLuint height)
{
    if (!Create2D(target, levels, format, width, height)) {
        return false;
    }

    MakeResident();

    return true;
}

// SubImage2D
void BindlessTexture::SubImage2D(GLenum target, GLuint level, GLuint xoffset, GLuint yoffset, GLuint width, GLuint height, GLenum format, GLenum type, GLvoid* data)
{
    glTextureSubImage2DEXT(m_texture, target, level, xoffset, yoffset, width, height, format, type, data);
}

// GenerateMipmap
void BindlessTexture::GenerateMipmap()
{
    glGenerateTextureMipmapEXT(m_texture, m_target);
}

// MakeResident
void BindlessTexture::MakeResident() const
{
    glMakeTextureHandleResidentNV(m_handle);
}

// MakeNonResident
void BindlessTexture::MakeNonResident() const
{
    glMakeTextureHandleNonResidentNV(m_handle);
}

// IsSupported
bool BindlessTexture::IsSupported()
{
    GLint numExtensions = 0;
    glGetIntegerv(GL_NUM_EXTENSIONS, &numExtensions);
    if (numExtensions == 0) {
        return false;
    }

    for (GLint extension = 0; extension < numExtensions; extension++) {
        if (strcmp((const char*)glGetStringi(GL_EXTENSIONS, extension), "GL_NV_bindless_texture") == 0) {
            return true;
        }
    }

    return false;
}

// Destroy
void BindlessTexture::Destroy()
{
    if (m_texture) {
        glDeleteTextures(1, &m_texture);
        m_texture = 0;
        m_handle = 0;
    }
}